@@ -5,6 +5,8 @@
     inclusive_range_methods,
 )]
 
+#![cfg_attr(feature = "cargo-clippy", allow(cast_lossless))]
+
 #[macro_use]
 extern crate log;
 
@@ -25,10 +27,13 @@ use rustc::ty::subst::Subst;
 use rustc::hir::def_id::DefId;
 use rustc::mir;
 
+use rustc_data_structures::fx::FxHasher;
+
 use syntax::ast::Mutability;
 use syntax::codemap::Span;
 
 use std::collections::{HashMap, BTreeMap};
+use std::hash::{Hash, Hasher};
 
 pub use rustc::mir::interpret::*;
 pub use rustc_mir::interpret::*;
@@ -131,107 +136,121 @@ impl ScalarExt for Scalar {
     }
 }
 
+pub fn create_ecx<'a, 'mir: 'a, 'tcx: 'mir>(
+    tcx: TyCtxt<'a, 'tcx, 'tcx>,
+    main_id: DefId,
+    start_wrapper: Option<DefId>,
+) -> EvalResult<'tcx, (EvalContext<'a, 'mir, 'tcx, Evaluator<'tcx>>, Option<Pointer>)> {
+    let mut ecx = EvalContext::new(
+        tcx.at(syntax::codemap::DUMMY_SP),
+        ty::ParamEnv::reveal_all(),
+        Default::default(),
+        MemoryData::new()
+    );
+
+    let main_instance = ty::Instance::mono(ecx.tcx.tcx, main_id);
+    let main_mir = ecx.load_mir(main_instance.def)?;
+    let mut cleanup_ptr = None; // Scalar to be deallocated when we are done
+
+    if !main_mir.return_ty().is_nil() || main_mir.arg_count != 0 {
+        return err!(Unimplemented(
+            "miri does not support main functions without `fn()` type signatures"
+                .to_owned(),
+        ));
+    }
+
+    if let Some(start_id) = start_wrapper {
+        let main_ret_ty = ecx.tcx.fn_sig(main_id).output();
+        let main_ret_ty = main_ret_ty.no_late_bound_regions().unwrap();
+        let start_instance = ty::Instance::resolve(
+            ecx.tcx.tcx,
+            ty::ParamEnv::reveal_all(),
+            start_id,
+            ecx.tcx.mk_substs(
+                ::std::iter::once(ty::subst::Kind::from(main_ret_ty)))
+            ).unwrap();
+        let start_mir = ecx.load_mir(start_instance.def)?;
+
+        if start_mir.arg_count != 3 {
+            return err!(AbiViolation(format!(
+                "'start' lang item should have three arguments, but has {}",
+                start_mir.arg_count
+            )));
+        }
+
+        // Return value
+        let size = ecx.tcx.data_layout.pointer_size;
+        let align = ecx.tcx.data_layout.pointer_align;
+        let ret_ptr = ecx.memory_mut().allocate(size, align, MemoryKind::Stack)?;
+        cleanup_ptr = Some(ret_ptr);
+
+        // Push our stack frame
+        ecx.push_stack_frame(
+            start_instance,
+            start_mir.span,
+            start_mir,
+            Place::from_ptr(ret_ptr, align),
+            StackPopCleanup::None,
+        )?;
+
+        let mut args = ecx.frame().mir.args_iter();
+
+        // First argument: pointer to main()
+        let main_ptr = ecx.memory_mut().create_fn_alloc(main_instance);
+        let dest = ecx.eval_place(&mir::Place::Local(args.next().unwrap()))?;
+        let main_ty = main_instance.ty(ecx.tcx.tcx);
+        let main_ptr_ty = ecx.tcx.mk_fn_ptr(main_ty.fn_sig(ecx.tcx.tcx));
+        ecx.write_value(
+            ValTy {
+                value: Value::Scalar(Scalar::Ptr(main_ptr)),
+                ty: main_ptr_ty,
+            },
+            dest,
+        )?;
+
+        // Second argument (argc): 1
+        let dest = ecx.eval_place(&mir::Place::Local(args.next().unwrap()))?;
+        let ty = ecx.tcx.types.isize;
+        ecx.write_scalar(dest, Scalar::from_u128(1), ty)?;
+
+        // FIXME: extract main source file path
+        // Third argument (argv): &[b"foo"]
+        let dest = ecx.eval_place(&mir::Place::Local(args.next().unwrap()))?;
+        let ty = ecx.tcx.mk_imm_ptr(ecx.tcx.mk_imm_ptr(ecx.tcx.types.u8));
+        let foo = ecx.memory.allocate_bytes(b"foo\0");
+        let ptr_size = ecx.memory.pointer_size();
+        let ptr_align = ecx.tcx.data_layout.pointer_align;
+        let foo_ptr = ecx.memory.allocate(ptr_size, ptr_align, MemoryKind::Stack)?;
+        ecx.memory.write_scalar(foo_ptr.into(), ptr_align, Scalar::Ptr(foo), ptr_size, false)?;
+        ecx.memory.mark_static_initialized(foo_ptr.alloc_id, Mutability::Immutable)?;
+        ecx.write_ptr(dest, foo_ptr.into(), ty)?;
+
+        assert!(args.next().is_none(), "start lang item has more arguments than expected");
+    } else {
+        ecx.push_stack_frame(
+            main_instance,
+            main_mir.span,
+            main_mir,
+            Place::from_scalar_ptr(Scalar::from_u128(1), ty::layout::Align::from_bytes(1, 1).unwrap()),
+            StackPopCleanup::None,
+        )?;
+
+        // No arguments
+        let mut args = ecx.frame().mir.args_iter();
+        assert!(args.next().is_none(), "main function must not have arguments");
+    }
+
+    Ok((ecx, cleanup_ptr))
+}
+
 pub fn eval_main<'a, 'tcx: 'a>(
     tcx: TyCtxt<'a, 'tcx, 'tcx>,
     main_id: DefId,
     start_wrapper: Option<DefId>,
 ) {
-    fn run_main<'a, 'mir: 'a, 'tcx: 'mir>(
-        ecx: &mut rustc_mir::interpret::EvalContext<'a, 'mir, 'tcx, Evaluator<'tcx>>,
-        main_id: DefId,
-        start_wrapper: Option<DefId>,
-    ) -> EvalResult<'tcx> {
-        let main_instance = ty::Instance::mono(ecx.tcx.tcx, main_id);
-        let main_mir = ecx.load_mir(main_instance.def)?;
-        let mut cleanup_ptr = None; // Scalar to be deallocated when we are done
-
-        if !main_mir.return_ty().is_nil() || main_mir.arg_count != 0 {
-            return err!(Unimplemented(
-                "miri does not support main functions without `fn()` type signatures"
-                    .to_owned(),
-            ));
-        }
-
-        if let Some(start_id) = start_wrapper {
-            let main_ret_ty = ecx.tcx.fn_sig(main_id).output();
-            let main_ret_ty = main_ret_ty.no_late_bound_regions().unwrap();
-            let start_instance = ty::Instance::resolve(
-                ecx.tcx.tcx,
-                ty::ParamEnv::reveal_all(),
-                start_id,
-                ecx.tcx.mk_substs(
-                    ::std::iter::once(ty::subst::Kind::from(main_ret_ty)))).unwrap();
-            let start_mir = ecx.load_mir(start_instance.def)?;
-
-            if start_mir.arg_count != 3 {
-                return err!(AbiViolation(format!(
-                    "'start' lang item should have three arguments, but has {}",
-                    start_mir.arg_count
-                )));
-            }
-
-            // Return value
-            let size = ecx.tcx.data_layout.pointer_size;
-            let align = ecx.tcx.data_layout.pointer_align;
-            let ret_ptr = ecx.memory_mut().allocate(size, align, Some(MemoryKind::Stack))?;
-            cleanup_ptr = Some(ret_ptr);
-
-            // Push our stack frame
-            ecx.push_stack_frame(
-                start_instance,
-                start_mir.span,
-                start_mir,
-                Place::from_ptr(ret_ptr, align),
-                StackPopCleanup::None,
-            )?;
-
-            let mut args = ecx.frame().mir.args_iter();
-
-            // First argument: pointer to main()
-            let main_ptr = ecx.memory_mut().create_fn_alloc(main_instance);
-            let dest = ecx.eval_place(&mir::Place::Local(args.next().unwrap()))?;
-            let main_ty = main_instance.ty(ecx.tcx.tcx);
-            let main_ptr_ty = ecx.tcx.mk_fn_ptr(main_ty.fn_sig(ecx.tcx.tcx));
-            ecx.write_value(
-                ValTy {
-                    value: Value::Scalar(Scalar::Ptr(main_ptr)),
-                    ty: main_ptr_ty,
-                },
-                dest,
-            )?;
-
-            // Second argument (argc): 1
-            let dest = ecx.eval_place(&mir::Place::Local(args.next().unwrap()))?;
-            let ty = ecx.tcx.types.isize;
-            ecx.write_scalar(dest, Scalar::from_u128(1), ty)?;
-
-            // FIXME: extract main source file path
-            // Third argument (argv): &[b"foo"]
-            let dest = ecx.eval_place(&mir::Place::Local(args.next().unwrap()))?;
-            let ty = ecx.tcx.mk_imm_ptr(ecx.tcx.mk_imm_ptr(ecx.tcx.types.u8));
-            let foo = ecx.memory.allocate_bytes(b"foo\0");
-            let ptr_size = ecx.memory.pointer_size();
-            let ptr_align = ecx.tcx.data_layout.pointer_align;
-            let foo_ptr = ecx.memory.allocate(ptr_size, ptr_align, None)?;
-            ecx.memory.write_scalar(foo_ptr.into(), ptr_align, Scalar::Ptr(foo), ptr_size, false)?;
-            ecx.memory.mark_static_initialized(foo_ptr.alloc_id, Mutability::Immutable)?;
-            ecx.write_ptr(dest, foo_ptr.into(), ty)?;
-
-            assert!(args.next().is_none(), "start lang item has more arguments than expected");
-        } else {
-            ecx.push_stack_frame(
-                main_instance,
-                main_mir.span,
-                main_mir,
-                Place::from_scalar_ptr(Scalar::from_u128(1), ty::layout::Align::from_bytes(1, 1).unwrap()),
-                StackPopCleanup::None,
-            )?;
-
-            // No arguments
-            let mut args = ecx.frame().mir.args_iter();
-            assert!(args.next().is_none(), "main function must not have arguments");
-        }
+    let (mut ecx, cleanup_ptr) = create_ecx(tcx, main_id, start_wrapper).expect("Couldn't create ecx");
 
+    let res: EvalResult = do catch {
         while ecx.step()? {}
         ecx.run_tls_dtors()?;
         if let Some(cleanup_ptr) = cleanup_ptr {
@@ -241,20 +260,38 @@ pub fn eval_main<'a, 'tcx: 'a>(
                 MemoryKind::Stack,
             )?;
         }
-        Ok(())
-    }
+    };
 
-    let mut ecx = EvalContext::new(tcx.at(syntax::codemap::DUMMY_SP), ty::ParamEnv::reveal_all(), Default::default(), Default::default());
-    match run_main(&mut ecx, main_id, start_wrapper) {
+    match res {
         Ok(()) => {
             let leaks = ecx.memory().leak_report();
             if leaks != 0 {
+                // TODO: Prevent leaks which aren't supposed to be there
                 //tcx.sess.err("the evaluated program leaked memory");
             }
         }
-        Err(mut e) => {
-            ecx.tcx.sess.err(&e.to_string());
-            ecx.report(&mut e, true, None);
+        Err(e) => {
+            if let Some(frame) = ecx.stack().last() {
+                let block = &frame.mir.basic_blocks()[frame.block];
+                let span = if frame.stmt < block.statements.len() {
+                    block.statements[frame.stmt].source_info.span
+                } else {
+                    block.terminator().source_info.span
+                };
+
+                let e = e.to_string();
+                let msg = format!("constant evaluation error: {}", e);
+                let mut err = struct_error(ecx.tcx.tcx.at(span), msg.as_str());
+                let (frames, span) = ecx.generate_stacktrace(None);
+                err.span_label(span, e);
+                for FrameInfo { span, location, .. } in frames {
+                    err.span_note(span, &format!("inside call to `{}`", location));
+                }
+                err.emit();
+            } else {
+                ecx.tcx.sess.err(&e.to_string());
+            }
+
             for (i, frame) in ecx.stack().iter().enumerate() {
                 trace!("-------------------");
                 trace!("Frame {}", i);
@@ -269,7 +306,7 @@ pub fn eval_main<'a, 'tcx: 'a>(
     }
 }
 
-#[derive(Default)]
+#[derive(Clone, Default, PartialEq, Eq)]
 pub struct Evaluator<'tcx> {
     /// Environment variables set by `setenv`
     /// Miri does not expose env vars from the host to the emulated program
@@ -279,15 +316,34 @@ pub struct Evaluator<'tcx> {
     pub(crate) suspended: HashMap<DynamicLifetime, Vec<ValidationQuery<'tcx>>>,
 }
 
+impl<'tcx> Hash for Evaluator<'tcx> {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        let Evaluator {
+            env_vars,
+            suspended: _,
+        } = self;
+
+        env_vars.iter()
+            .map(|(env, ptr)| {
+                let mut h = FxHasher::default();
+                env.hash(&mut h);
+                ptr.hash(&mut h);
+                h.finish()
+            })
+            .fold(0u64, |acc, hash| acc.wrapping_add(hash))
+            .hash(state);
+    }
+}
+
 pub type TlsKey = u128;
 
-#[derive(Copy, Clone, Debug)]
+#[derive(Copy, Clone, Debug, Hash, PartialEq, Eq)]
 pub struct TlsEntry<'tcx> {
     data: Scalar, // Will eventually become a map from thread IDs to `Scalar`s, if we ever support more than one thread.
     dtor: Option<ty::Instance<'tcx>>,
 }
 
-#[derive(Default)]
+#[derive(Clone, PartialEq, Eq)]
 pub struct MemoryData<'tcx> {
     /// The Key to use for the next thread-local allocation.
     next_thread_local: TlsKey,
@@ -304,6 +360,30 @@ pub struct MemoryData<'tcx> {
     statics: HashMap<GlobalId<'tcx>, AllocId>,
 }
 
+impl<'tcx> MemoryData<'tcx> {
+    fn new() -> Self {
+        MemoryData {
+            next_thread_local: 1, // start with 1 as we must not use 0 on Windows
+            thread_local: BTreeMap::new(),
+            locks: HashMap::new(),
+            statics: HashMap::new(),
+        }
+    }
+}
+
+impl<'tcx> Hash for MemoryData<'tcx> {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        let MemoryData {
+            next_thread_local: _,
+            thread_local,
+            locks: _,
+            statics: _,
+        } = self;
+
+        thread_local.hash(state);
+    }
+}
+
 impl<'mir, 'tcx: 'mir> Machine<'mir, 'tcx> for Evaluator<'tcx> {
     type MemoryData = MemoryData<'tcx>;
     type MemoryKinds = memory::MemoryKind;
@@ -379,7 +459,7 @@ impl<'mir, 'tcx: 'mir> Machine<'mir, 'tcx> for Evaluator<'tcx> {
         let ptr = ecx.memory.allocate(
             layout.size,
             layout.align,
-            None,
+            MemoryKind::Stack,
         )?;
 
         // Step 4: Cache allocation id for recursive statics
@@ -402,14 +482,11 @@ impl<'mir, 'tcx: 'mir> Machine<'mir, 'tcx> for Evaluator<'tcx> {
                 let frame = ecx.frame_mut();
                 let bb = &frame.mir.basic_blocks()[frame.block];
                 if bb.statements.len() == frame.stmt && !bb.is_cleanup {
-                    match bb.terminator().kind {
-                        ::rustc::mir::TerminatorKind::Return => {
-                            for (local, _local_decl) in mir.local_decls.iter_enumerated().skip(1) {
-                                // Don't deallocate locals, because the return value might reference them
-                                frame.storage_dead(local);
-                            }
+                    if let ::rustc::mir::TerminatorKind::Return = bb.terminator().kind {
+                        for (local, _local_decl) in mir.local_decls.iter_enumerated().skip(1) {
+                            // Don't deallocate locals, because the return value might reference them
+                            frame.storage_dead(local);
                         }
-                        _ => {}
                     }
                 }
             }
@@ -453,7 +530,7 @@ impl<'mir, 'tcx: 'mir> Machine<'mir, 'tcx> for Evaluator<'tcx> {
                 value: Value::Scalar(Scalar::from_u128(match layout.size.bytes() {
                     0 => 1 as u128,
                     size => size as u128,
-                }.into())),
+                })),
                 ty: usize,
             },
             dest,
